@@ -1,9 +1,11 @@
 /* Shuffling, bootstrapping, permuting alignments, by column or row.
  * 
  * Table of contents:
- *    1. Randomizing MSAs by column.
- *    2. Permuting sequence order (i.e. by row)
- *    3. Shuffling pairwise (QRNA) alignments.
+ *    1. Shuffling or resampling columns ("horizontal" shuffling)
+ *    2. Shuffling residues within columns ("vertical" shuffling)
+ *    3. Permuting sequence order (i.e. by row)
+ *    4. Shuffling pairwise (QRNA) alignments.
+ *    5. Example
  */
 #include "esl_config.h"
 
@@ -17,7 +19,7 @@
 #include "esl_msashuffle.h"
 
 /*****************************************************************
- * 1. Randomizing MSAs by column.
+ * 1. Shuffling or resampling columns ("horizontal" shuffling)
  *****************************************************************/ 
 
 /* Function:  esl_msashuffle_Shuffle()
@@ -66,7 +68,7 @@ esl_msashuffle_Shuffle(ESL_RANDOMNESS *r, ESL_MSA *msa, ESL_MSA *shuf)
 	  pos = esl_rnd_Roll(r, alen);
 	  for (i = 0; i < msa->nseq; i++)
 	    {
-	      c                     = msa->aseq[i][pos];
+	      c                     = shuf->aseq[i][pos];
 	      shuf->aseq[i][pos]    = shuf->aseq[i][alen-1];
 	      shuf->aseq[i][alen-1] = c;
 	    }
@@ -90,7 +92,7 @@ esl_msashuffle_Shuffle(ESL_RANDOMNESS *r, ESL_MSA *msa, ESL_MSA *shuf)
 	  pos = esl_rnd_Roll(r, alen) + 1;
 	  for (i = 0; i < msa->nseq; i++)
 	    {
-	      x                 = msa->ax[i][pos];
+	      x                 = shuf->ax[i][pos];
 	      shuf->ax[i][pos]  = shuf->ax[i][alen];
 	      shuf->ax[i][alen] = x;
 	    }
@@ -170,7 +172,76 @@ esl_msashuffle_Bootstrap(ESL_RANDOMNESS *r, ESL_MSA *msa, ESL_MSA *bootsample)
 
 
 /*****************************************************************
- * 2. Permuting the sequence order 
+ * 2. Shuffling residues within columns ("vertical" shuffling)
+ *****************************************************************/
+
+/* Function:  esl_msashuffle_VShuffle()
+ * Synopsis:  Shuffle <msa> residues within independent columns
+ * Incept:    SRE, Tue 10 Jul 2018 [World Cup, France v. Belgium]
+ *
+ * Purpose:   Shuffle the residues in each column of <msa>
+ *            independently, using random generator <rng>. 
+ *            Return the shuffled alignment in <shuf>, space
+ *            allocated by the caller. <msa> and <shuf> can
+ *            be identical to shuffle <msa> in place.
+ *
+ *            Caller is responsible for the metadata in <shuf> (name,
+ *            etc.; everything but the alignment itself).  Easiest
+ *            thing for caller to do is to <esl_msa_Clone()> the <msa>
+ *            to create <shuf>, then shuffle it.
+ *
+ *            <msa> and <shuf> must be in digital mode.
+ *
+ * Args:      rng  - random number generator
+ *            msa  - input multiple alignment to shuffle
+ *            shuf - RESULT: shuffled <msa>
+ *
+ * Returns:   <eslOK> on success, and <shuf> contains the shuffled
+ *            alignment.
+ *
+ * Throws:    <eslEMEM> on allocation error. Now <shuf> is untouched.
+ */
+int
+esl_msashuffle_VShuffle(ESL_RANDOMNESS *rng, const ESL_MSA *msa, ESL_MSA *shuf)
+{
+  ESL_DASSERT1 ((  msa->flags  & eslMSA_DIGITAL ));
+  ESL_DASSERT1 ((  shuf->flags & eslMSA_DIGITAL ));
+  ESL_DSQ *csq = NULL;
+  int      idx, apos;
+  int      nres;      // number of non-gap residues in this column
+  int      status;
+
+
+  ESL_ALLOC(csq, sizeof(ESL_DSQ) * (msa->nseq+2));  // +2 because we hack <csq> column to look like a digital sequence, suitable for esl_rsq*
+  csq[0] = eslDSQ_SENTINEL;
+  
+  for (apos = 1; apos <= msa->alen; apos++)
+    {
+      /* transpose each column from [idx][apos] to an array (residues only) we can shuffle */
+      for (idx = 0, nres = 0; idx < msa->nseq; idx++)
+	if (! esl_abc_XIsGap(msa->abc, msa->ax[idx][apos]))
+	  csq[++nres] = msa->ax[idx][apos];  // (again, the prepend ++nres here is because we make <csq> look like a digital seq, starts at 1
+      csq[nres+1] = eslDSQ_SENTINEL;
+
+      /* shuffle it (remember, it's only the residues (and *,~), not the gaps */
+      esl_rsq_XShuffle(rng, csq, nres, csq);
+
+      /* put it back in <shuf> */
+      for (idx = 0, nres = 0; idx < msa->nseq; idx++)
+	if (! esl_abc_XIsGap(msa->abc, msa->ax[idx][apos]))
+	  shuf->ax[idx][apos] = csq[++nres];   // (again the ++, so we start at 1)
+    }
+  free(csq);
+  return eslOK;
+
+ ERROR:
+  return status;
+}  
+
+
+
+/*****************************************************************
+ * 3. Permuting the sequence order 
  *****************************************************************/
 
 /* Function:  esl_msashuffle_PermuteSequenceOrder()
@@ -227,7 +298,7 @@ esl_msashuffle_PermuteSequenceOrder(ESL_RANDOMNESS *r, ESL_MSA *msa)
 
 
 /*****************************************************************
- * 3. Shuffling pairwise (QRNA) alignments
+ * 4. Shuffling pairwise (QRNA) alignments
  *****************************************************************/ 
 
 /* Function: esl_msashuffle_XQRNA()
@@ -422,8 +493,10 @@ esl_msashuffle_CQRNA(ESL_RANDOMNESS *r, ESL_ALPHABET *abc, char *x, char *y, cha
 }
 
 
+
+
 /*****************************************************************
- * 4. Example.
+ * 5. Example.
  *****************************************************************/
 #ifdef eslMSASHUFFLE_EXAMPLE
 #include <stdio.h>
