@@ -34,7 +34,7 @@ static ESL_SQ *sq_create_from(const char *name, const char *desc, const char *ac
 static ESL_SQ_BLOCK *sq_createblock(int count, int do_digital);
 
 static int  sq_init(ESL_SQ *sq, int do_digital);
-static void sq_free(ESL_SQ *sq);
+
 
 /*****************************************************************
  *# 1. Text version of the <ESL_SQ> object.
@@ -509,6 +509,37 @@ esl_sq_IsText(const ESL_SQ *sq)
 }
 
 
+/* sq_free_internals()
+ * Free the insides of an <ESL_SQ> but not the shell.
+ * We need this version in a ESL_SQ_BLOCK, which allocates
+ * an array of ESL_SQ structures, not one at a time.
+ */
+static void
+sq_free_internals(ESL_SQ *sq)
+{
+  if (sq)
+    {
+      int   x;        /* index for optional extra residue markups */
+      free(sq->name);  
+      free(sq->acc);   
+      free(sq->desc);  
+      free(sq->seq);   
+      free(sq->dsq);   
+      free(sq->ss);    
+      free(sq->source);
+      if (sq->nxr) {  
+	for (x = 0; x < sq->nxr; x++) {
+	  free(sq->xr[x]);
+	  free(sq->xr_tag[x]);
+	}
+      }
+      free(sq->xr);
+      free(sq->xr_tag);
+    }
+  return;
+}
+
+
 /* Function:  esl_sq_Destroy()
  * Synopsis:  Frees an <ESL_SQ>.
  * Incept:    SRE, Thu Dec 23 12:28:07 2004 [Zaragoza]
@@ -518,31 +549,17 @@ esl_sq_IsText(const ESL_SQ *sq)
 void
 esl_sq_Destroy(ESL_SQ *sq)
 {
-  int   x;        /* index for optional extra residue markups */
-  if (sq == NULL) return;
-
-  if (sq->name   != NULL) free(sq->name);  
-  if (sq->acc    != NULL) free(sq->acc);   
-  if (sq->desc   != NULL) free(sq->desc);  
-  if (sq->seq    != NULL) free(sq->seq);   
-  if (sq->dsq    != NULL) free(sq->dsq);   
-  if (sq->ss     != NULL) free(sq->ss);    
-  if (sq->source != NULL) free(sq->source);
-  if (sq->nxr > 0) {  
-    for (x = 0; x < sq->nxr; x++) {
-      if (sq->xr[x]     != NULL) free(sq->xr[x]);
-      if (sq->xr_tag[x] != NULL) free(sq->xr_tag[x]);
+  if (sq)
+    {
+      sq_free_internals(sq);
+      free(sq);
     }
-    if (sq->xr     != NULL) free(sq->xr);
-    if (sq->xr_tag != NULL) free(sq->xr_tag);
-  }
-  free(sq);
   return;
 }
 
+
 /* Function:  esl_sq_CreateBlock()
  * Synopsis:  Create a new block of empty <ESL_SQ>.
- * Incept:    
  *
  * Purpose:   Creates a block of empty <ESL_SQ> sequence objects.
  *            
@@ -559,7 +576,6 @@ esl_sq_CreateBlock(int count)
 
 /* Function:  esl_sq_DestroyBlock()
  * Synopsis:  Frees an <ESL_SQ_BLOCK>.
- * Incept:    
  *
  * Purpose:   Free a Create()'d block of <sq>.
  */
@@ -571,23 +587,53 @@ esl_sq_DestroyBlock(ESL_SQ_BLOCK *block)
   if (block == NULL) return;
 
   for (i = 0; i < block->listSize; ++i)
-    {
-      sq_free(block->list + i);
-    }
+    sq_free_internals(block->list + i);
 
   free(block->list);
   free(block);
   return;
 }
 
+/* Function: esl_sq_BlockReallocSequences   
+ * Synopsis: Re-allocates the internal data structures of a block's sequences back to their defaults
+ *
+ * Purpose: In some uses of ESL_SQ_BLOCK, fetching long sequences into the block causes the block to grow
+ *          to unacceptable sizes.  This function reallocates the internal data structures of each of a 
+ *          block's sequences back to their default values to shrink the block down to a reasonable size. 
+ *          This destroy's the sequences, so only call this function when the block's contents aren't needed any more.
+ * 
+ * Returns: <eslOK> on success
+ *
+ * Throws: <eslEMEM> on allocation failure
+ */
+int esl_sq_BlockReallocSequences(ESL_SQ_BLOCK *block){  
+  int status;
+  for(int i = 0; i < block->listSize; i++){
+    (block->list+i)->nalloc   = eslSQ_NAMECHUNK; 
+    (block->list+i)->aalloc   = eslSQ_ACCCHUNK;
+    (block->list+i)->dalloc   = eslSQ_DESCCHUNK;
+    (block->list+i)->salloc   = eslSQ_SEQCHUNK; 
+    (block->list+i)->srcalloc = eslSQ_NAMECHUNK; 
+    ESL_REALLOC((block->list+i)->name,   sizeof(char) * (block->list+i)->nalloc);
+    ESL_REALLOC((block->list+i)->acc,    sizeof(char) * (block->list+i)->aalloc);
+    ESL_REALLOC((block->list+i)->desc,   sizeof(char) * (block->list+i)->dalloc);
+    ESL_REALLOC((block->list+i)->source, sizeof(char) * (block->list+i)->srcalloc);
+    if ((block->list+i)->dsq!=NULL) ESL_REALLOC((block->list+i)->dsq,  sizeof(ESL_DSQ) * (block->list+i)->salloc);
+    else            ESL_REALLOC((block->list+i)->seq,  sizeof(char)    * (block->list+i)->salloc);
+    if ((block->list+i)->ss != NULL){
+      ESL_REALLOC((block->list+i)->ss,  sizeof(char)    * (block->list+i)->salloc);
+    }
+  } 
+  return(eslOK);  
+ERROR:  
+  return(status);
+}
+
 /* Function:  esl_sq_BlockGrowTo()
  * Synopsis:  Grows a sequence block to hold at least <n> <ESL_SQ>.
- * Incept:    
  *
- * Purpose:   Assure that the list of sequences
- *            can hold up to a total of <n> sequences,
- *            reallocating as needed.
- *            
+ * Purpose:   Assure that the list of sequences can hold up to a total
+ *            of <n> sequences, reallocating as needed.
  *
  * Returns:   <eslOK> on success.
  *
@@ -1464,11 +1510,14 @@ esl_sq_XAddResidue(ESL_SQ *sq, ESL_DSQ x)
  *            data). Degenerate nucleic acid IUPAC characters are
  *            complemented appropriately.
  *
- *            The <start/end> coords in <sq> are swapped. (Note that
- *            in the unusual case of sequences of length 1,
- *            <start=end> and we can't unambiguously tell if a seq is
- *            in the reverse complement direction or not; this is a
- *            minor flaw in Easel's current coordinate handling.)
+ *            The <start/end> coords in <sq> are swapped in the
+ *            source-tracking coordinate info. If <sq> is length 1,
+ *            we're unable to unambiguously differentiate forward
+ *            vs. reverse strand in Easel's
+ *            1-offset/closed-interval/no-flag coord system (see
+ *            comments in esl_sq.h), so caller needs to hack some way
+ *            of remembering for itself that the sequence is reverse
+ *            complemented.
  *
  * Returns:   <eslOK> on success.
  *            
@@ -1545,6 +1594,7 @@ esl_sq_ReverseComplement(ESL_SQ *sq)
     }
 
   ESL_SWAP(sq->start, sq->end, int64_t);
+
   /* revcomp invalidates any secondary structure annotation */
   if (sq->ss != NULL) { free(sq->ss); sq->ss = NULL; }
   /* revcomp invalidates any extra residue markup */
@@ -1554,7 +1604,6 @@ esl_sq_ReverseComplement(ESL_SQ *sq)
     free(sq->xr_tag); sq->xr_tag = NULL;
     free(sq->xr);     sq->xr     = NULL;
   }   
-  
   return status;
 
  ERROR:
@@ -2137,8 +2186,15 @@ sq_init(ESL_SQ *sq, int do_digital)
   ESL_ALLOC(sq->acc,    sizeof(char) * sq->aalloc);
   ESL_ALLOC(sq->desc,   sizeof(char) * sq->dalloc);
   ESL_ALLOC(sq->source, sizeof(char) * sq->srcalloc);
-  if (do_digital) ESL_ALLOC(sq->dsq,  sizeof(ESL_DSQ) * sq->salloc);
-  else            ESL_ALLOC(sq->seq,  sizeof(char)    * sq->salloc);
+  if (do_digital)
+    {
+      ESL_ALLOC(sq->dsq,  sizeof(ESL_DSQ) * sq->salloc);
+    }
+  else 
+    {
+      ESL_ALLOC(sq->seq,  sizeof(char)    * sq->salloc);
+      sq->abc = NULL;
+    }
 
   esl_sq_Reuse(sq);	/* initialization of sq->n, offsets, and strings */
   return eslOK;
@@ -2235,28 +2291,6 @@ sq_create_from(const char *name, const char *desc, const char *acc)
   return NULL;
 }
 
-/* Free <ESL_SQ> object */
-static void
-sq_free(ESL_SQ *sq)
-{
-  int   x;        /* index for optional extra residue markups */
-  if (sq->name   != NULL)   free(sq->name);
-  if (sq->acc    != NULL)   free(sq->acc);
-  if (sq->desc   != NULL)   free(sq->desc);
-  if (sq->source != NULL)   free(sq->source);
-  if (sq->seq    != NULL)   free(sq->seq);
-  if (sq->dsq    != NULL)   free(sq->dsq);
-  if (sq->ss     != NULL)   free(sq->ss);
-  if (sq->nxr > 0) {
-    for (x = 0; x < sq->nxr; x++) {
-      if (sq->xr[x]     != NULL) free(sq->xr[x]);
-      if (sq->xr_tag[x] != NULL) free(sq->xr_tag[x]);  
-    }       
-    if (sq->xr     != NULL) free(sq->xr);
-    if (sq->xr_tag != NULL) free(sq->xr_tag); 
-  }    
-}  
-
 /*----------------- end, internal functions ---------------------*/
 
 
