@@ -28,7 +28,8 @@
 #define P7_HMMERH_INCLUDED
 #include "p7_config.h"
 
-#include <stdio.h>		/* FILE */
+#include <stdio.h>		
+#include <stddef.h>             // ptrdiff_t 
 
 #ifdef HMMER_MPI
 #include "mpi.h"
@@ -612,16 +613,16 @@ typedef struct p7_alidisplay_s {
 typedef struct p7_dom_s { 
   int64_t        ienv, jenv;
   int64_t        iali, jali;
-  int64_t        iorf, jorf; /*Used in translated search to capture the range in the DNA sequence of the ORF containing the match to a protein query */
-  float          envsc;  	/* Forward score in envelope ienv..jenv; NATS; without null2 correction       */
-  float          domcorrection;	/* null2 score when calculating a per-domain score; NATS                      */
-  float          dombias;	/* FLogsum(0, log(bg->omega) + domcorrection): null2 score contribution; NATS */
-  float          oasc;		/* optimal accuracy score (units: expected # residues correctly aligned)      */
-  float          bitscore;	/* overall score in BITS, null corrected, if this were the only domain in seq */
-  double         lnP;	        /* log(P-value) of the bitscore                                               */
-  int            is_reported;	/* TRUE if domain meets reporting thresholds                                  */
-  int            is_included;	/* TRUE if domain meets inclusion thresholds                                  */
-  float         *scores_per_pos; /* score in BITS that each position in the alignment contributes to an overall viterbi score */
+  int64_t        iorf, jorf;     /* Used in translated search to capture the range in the DNA sequence of the ORF containing the match to a protein query */
+  float          envsc;  	 /* Forward score in envelope ienv..jenv; NATS; without null2 correction       */
+  float          domcorrection;	 /* null2 score when calculating a per-domain score; NATS                      */
+  float          dombias;	 /* FLogsum(0, log(bg->omega) + domcorrection): null2 score contribution; NATS */
+  float          oasc;		 /* optimal accuracy score (units: expected # residues correctly aligned)      */
+  float          bitscore;	 /* overall score in BITS, null corrected, if this were the only domain in seq */
+  double         lnP;	         /* log(P-value) of the bitscore                                               */
+  int            is_reported;	 /* TRUE if domain meets reporting thresholds                                  */
+  int            is_included;	 /* TRUE if domain meets inclusion thresholds                                  */
+  float         *scores_per_pos; /* only used by `nhmmer --aliscoresout`; score in BITS that each pos in ali contributes to viterbi score */
   P7_ALIDISPLAY *ad; 
 } P7_DOMAIN;
 
@@ -1183,10 +1184,9 @@ typedef struct p7_pipeline_s {
   /* Domain postprocessing                                                  */
   ESL_RANDOMNESS *r;		/* random number generator                  */
   int             do_reseeding; /* TRUE: reseed for reproducible results    */
-  int             do_alignment_score_calc;
+  int  do_alignment_score_calc; /* used only by nhmmer --aliscoresout       */
   P7_DOMAINDEF   *ddef;		/* domain definition workflow               */
 
-
   /* Reporting threshold settings                                           */
   int     by_E;		        /* TRUE to cut per-target report off by E   */
   double  E;	                /* per-target E-value threshold             */
