@@ -424,7 +424,7 @@ p7_Builder(P7_BUILDER *bld, ESL_MSA *msa, P7_BG *bg,
   if ((status =  validate_msa         (bld, msa))                       != eslOK) goto ERROR;
   if ((status =  esl_msa_Checksum     (msa, &checksum))                 != eslOK) ESL_XFAIL(status, bld->errbuf, "Failed to calculate checksum"); 
   if ((status =  relative_weights     (bld, msa))                       != eslOK) goto ERROR;
-  if ((status =  esl_msa_MarkFragments(msa, bld->fragthresh))           != eslOK) goto ERROR;
+  if ((status =  esl_msa_MarkFragments_old(msa, bld->fragthresh))           != eslOK) goto ERROR;
   if ((status =  build_model          (bld, msa, &hmm, tr_ptr))         != eslOK) goto ERROR;
 
 
@@ -481,7 +481,7 @@ p7_Builder(P7_BUILDER *bld, ESL_MSA *msa, P7_BG *bg,
  *            
  * Args:      bld       - build configuration
  *            sq        - query sequence
- *            bg        - null model (needed to paramaterize insert emission probs)
+ *            bg        - null model (needed to parameterize insert emission probs)
  *            opt_hmm   - optRETURN: new HMM
  *            opt_gm    - optRETURN: profile corresponding to <hmm>
  *            opt_om    - optRETURN: optimized profile corresponding to <gm>
@@ -774,7 +774,7 @@ p7_Builder_MaxLength (P7_HMM *hmm, double emit_thresh)
  * 
  * HMMER uses a convention for missing data characters: they
  * indicate that a sequence is a fragment.  (See
- * esl_msa_MarkFragments()).
+ * esl_msa_MarkFragments_old()).
  *
  * Because of the way these fragments will be handled in tracebacks,
  * we reject any alignment that uses missing data characters in any
@@ -807,16 +807,20 @@ validate_msa(P7_BUILDER *bld, ESL_MSA *msa)
 static int
 relative_weights(P7_BUILDER *bld, ESL_MSA *msa)
 {
-  int status = eslOK;
+  ESL_MSAWEIGHT_CFG *cfg    = esl_msaweight_cfg_Create();
+  int                status = eslOK;
+
+  cfg->ignore_rf = (bld->arch_strategy == p7_ARCH_HAND ? FALSE : TRUE);  // PB weights use RF consensus columns only if --hand is set. [iss #180]
 
   if      (bld->wgt_strategy == p7_WGT_NONE)                    { esl_vec_DSet(msa->wgt, msa->nseq, 1.); }
   else if (bld->wgt_strategy == p7_WGT_GIVEN)                   ;
-  else if (bld->wgt_strategy == p7_WGT_PB)                      status = esl_msaweight_PB(msa); 
+  else if (bld->wgt_strategy == p7_WGT_PB)                      status = esl_msaweight_PB_adv(cfg, msa, /*ESL_MSAWEIGHT_DAT=*/ NULL); 
   else if (bld->wgt_strategy == p7_WGT_GSC)                     status = esl_msaweight_GSC(msa); 
   else if (bld->wgt_strategy == p7_WGT_BLOSUM)                  status = esl_msaweight_BLOSUM(msa, bld->wid); 
   else ESL_EXCEPTION(eslEINCONCEIVABLE, "no such weighting strategy");
 
   if (status != eslOK) ESL_FAIL(status, bld->errbuf, "failed to set relative weights in alignment");
+  esl_msaweight_cfg_Destroy(cfg);
   return eslOK;
 }
 
